#include "director.h"

#include "galaxy.h"
#include "logging.h"
#include "utils.h"

Director::Director() {
  BUGZ_LOG(warning) << "Director::Director()";

  // active = false;
  game = 0;  // not in a game
  proxy_deactivate();
}

Director::~Director() { BUGZ_LOG(warning) << "Director::~Director()"; }

void Director::client_input(const std::string &input) {
  // If we're already active, don't try to activate.

  if (!active && (input == "\x1b" || input == "~")) {
    std::string &prompt = current_prompt; 
    BUGZ_LOG(trace) << "CI: ACTIVATE prompt shows: [" << prompt << "]";

    if (prompt == "Selection (? for menu): ") {
      to_client(
          "\n\rThere's not much we can do here.  Activate in-game at a "
          "Command prompt.\n\r");
      to_client(current_raw_prompt);
      return;
    }

    // easter-eggs:

    if (prompt == "Enter your choice: ") {
      to_client(
          "\n\r\x1b[1;36mI'd choose \x1b[1;37m`T`\x1b[1;36m, but "
          "that's how I was coded.\n\r");
      to_client(current_raw_prompt);
      return;
    }

    // easter-egg
    if (prompt == "[Pause]") {
      to_client(" \x1b[1;36mPAWS\x1b[0m\n\r");
      to_client(current_raw_prompt);
      return;
    }

    if (prompt == "Planet command (?=help) [D] ") {
      // future:  Activate at planet menu ?
      return;
    }
    //
    // The command prompt that we're looking for:
    //
    // "Command [TL=00:00:00]:[242] (?=Help)? : "
    // the time, and the sector number vary...

    if (prompt.substr(0, 9) == "Command [") {
      int len = prompt.length();
      if (prompt.substr(len - 14) == "] (?=Help)? : ") {
        proxy_activate();
        /*
        to_client("\n\r\x1b[1;34mWELCOME!  This is where the proxy would "
                  "activate.\n\r");
        // active = true;
        // show_client = true;  // because if something comes (unexpected)
        // from the server? talk_direct = false;

        // but we aren't activating (NNY)
        to_client(get_prompt());
        */
        return;
      }
    }
  }
  // Ok...
  if (talk_direct) to_server(input);

  /*
  if (emit_client_input)
    emit_client_input(line);
  */
}

void Director::server_line(const std::string &line) {
  // check for if we entered game/left game

  if (line.find("TradeWars Game Server   ") != std::string::npos) {
    to_client("\rTradeWars Proxy v2++ READY (~ or ESC to activate)\n\r");
    game = 0;
    // reset "active game" -- we're back at the menu
  }

  if (line.find("Selection (? for menu): ") != std::string::npos) {
    char ch = line[line.length() - 1];
    if (ch >= 'A' && ch < 'Q') {
      game = ch;
      BUGZ_LOG(warning) << "GAME " << game << " activated!";
    }
    // not needed (handled by above Game Server check).
    if (ch == 'Q') game = 0;
  }

  if (game) {
    // in-game parsing here.
  }

  /*
  if (emit_server_line) {
    emit_server_line(line);
  }
   */
}

void Director::server_prompt(const std::string &prompt, const std::string &raw_prompt) {
  current_prompt = prompt;
  current_raw_prompt = raw_prompt;
  /*
  if (emit_server_prompt)
    emit_server_prompt(prompt);
   */
}

void Director::proxy_activate(void) {
  active = true;  // sets Session keep-alive timer.
  // set other values we need
  talk_direct = false;
  show_client = false;
}

void Director::proxy_deactivate(void) {
  active = false;
  // reset everything back to good state
  talk_direct = true;
  show_client = true;
}

void Director::SL_cimline(const std::string &line) {
  if (line == ": ENDINTERROG") {
    SL_parser = nullptr;
    return;
  }
  if (line == ": ") {
    // do I need to do anything special here for this?
    return;
  }
  if (line.empty()) {
    SL_parser = nullptr;
    return;
  }

  // parse cimline
  size_t pos = line.find('%');
  std::string work = line;

  if (pos == line.npos) {
    // warpcim
    BUGZ_LOG(fatal) << "warpcim: [" << line << "]";
    auto warps = split(line);
    sector_warps sw;
    for (auto const &w : warps) {
      if (sw.sector == 0) {
        sw.sector = stoi(w);
      } else {
        sw.add(stoi(w));
      }
    }
    BUGZ_LOG(fatal) << "warpcim: " << sw;

  } else {
    // portcim
    struct port p = parse_portcim(line);
    if (p.sector == 0)
      BUGZ_LOG(fatal) << "portcim: [" << line << "]";
    else
      BUGZ_LOG(fatal) << "portcim: " << p;
  }
}

void Director::SL_thiefline(const std::string &line) {
  size_t pos = line.find("Suddenly you're Busted!");
  bool busted = pos != line.npos;
  if (busted) {
    BUGZ_LOG(fatal) << "set bust";
    SL_parser = nullptr;
  } else {
    pos = line.find("(You realize the guards saw you last time!)");
    if (pos != line.npos) SL_parser = nullptr;
  }

  // Are those the two ways to exit from this state?
}
void Director::SL_sectorline(const std::string &line) {
  BUGZ_LOG(fatal) << "sectorline: [" << line << "]";
}

void Director::SL_portline(const std::string &line) {
  if (line.empty()) {
    SL_parser = nullptr;
    return;
  }
  BUGZ_LOG(info) << "portline : " << line;
  size_t pos = line.find('%');
  if (pos != line.npos) {
    // Ok, this is a valid portline
    std::string work = line;
    replace(work, "Fuel Ore", "Fuel");
    BUGZ_LOG(fatal) << "re.split? : [" << work << "]";
  }
}

void Director::SL_warpline(const std::string &line) {
  if (line.empty()) {
    SL_parser = nullptr;
    return;
  }

  // process warp line
  BUGZ_LOG(fatal) << "warpline: [" << line << "]";
}